
package com.topcoder.message.im.connection;
import com.topcoder.message.im.Connection;

/**
 * <p>AbstractSocketConnection is the base class for both GenericSocketConnection and DirectSocketConnection. It implements most of the method constract from Connection interface to minimize code redundancy. The only difference on the implementation will be handled by initializeConnection() and sendMessage(). Implementation can override closeConnection() to add clear up code. But they should use super.closeConnection() to invoke the clear up code on the base class.</p>
 * <p>Methods should synchronize on the class instance.</p>
 * 
 */
public abstract class AbstractSocketConnection {

/**
 * <p>The key to read the listening port from the parameters. The attached value could either be a string or an Integer.</p>
 * 
 */
    public static final String LISTENING_PORT_PARAM_KEY = "listening_port";

/**
 * <p>Whether the connection is open. It will be handled by open/close methods. Initialized to false.</p>
 * 
 */
    private boolean connectionOpen = false;

/**
 * <p>The default parameters from constructor. Simply store there and is immutable.</p>
 * 
 */
    private java.util.Map defaultParameters = null;

/**
 * <p>The listener of the connection that handles incoming messages. It is initialized in the openConnection() methods and is mutable with protected getter and setter.</p>
 * 
 */
    private com.topcoder.message.im.connection.MessageListener listener = null;
/**
 * <p></p>
 */
    public com.topcoder.message.im.connection.MessageListener messageListener;

/**
 * <p>Creates AbstractSocketConnection with default parameters. Save a shallow copy of the parameter mapping to the defaultParameters field.</p>
 * 
 * 
 * 
 * 
 * @param parameters the initialization parameters
 * @exception NullPointerException if parameters is null
 */
    protected  AbstractSocketConnection(java.util.Map parameters) {        
        // your code here
    } 

/**
 * <p>Open connection with default parameters and default processor (MessageQueue). It will try to initialize a message listener if it is null, and invoke initializeConnection() with default parameters. isConnectionOpen is set to true. If connection is already open, nothing is performed and ConnectionStateException is thrown.</p>
 * 
 * 
 * 
 * 
 * @exception ConnectionStateException if connection is already open
 * @exception ConnectionInitializationException if initialization fails
 */
    public void openConnection() {        
        // your code here
    } 

/**
 * <p>Open connection with default parameters merged with new parameters and named processor. It will try to initialize a message listener if it is null, and invoke initializeConnection() with parameters. The parameter merging should override parameters in the default mapping. isConnectionOpen is set to true. If connection is already open, nothing is performed and ConnectionStateException is thrown.</p>
 * 
 * 
 * 
 * 
 * @param parameters the parameters to initialize the connection
 * @param processor the message processor the handle the incoming messages
 * @exception NullPointerException if parameters or processor is null
 * @exception ConnectionStateException if connection is already open
 * @exception ConnectionInitializationException if initialization fails
 */
    public void openConnection(java.util.Map parameters, com.topcoder.message.im.MessageProcessor processor) {        
        // your code here
    } 

/**
 * <p>Close the connection. It will close the listening socket with MessageListener.close(). This method could be overriden by subclassing, but they should still reference the super closeConnection() method to clear up the listener. connectionOpen is set to false.</p>
 * 
 * 
 * 
 */
    public void closeConnection() {        
        // your code here
    } 

/**
 * <p>Check whether connection is open. Simply return the connectionOpen field.</p>
 * 
 * 
 * 
 * 
 * @return whether the connection is open
 */
    public boolean isConnectionOpen() {        
        // your code here
        return false;
    } 

/**
 * <p>Get the message processor for this connection. If listener is null, it simply returns null. Otherwise listener.getMessageProcessor() is returned.</p>
 * 
 * 
 * 
 * 
 * @return the message processor for the connection
 */
    public com.topcoder.message.im.MessageProcessor getMessageProcessor() {        
        // your code here
        return null;
    } 

/**
 * <p>Check whether the message can be handled by the connection. For the socket implementation they just accept anything.</p>
 * 
 * 
 * 
 * 
 * @return true
 * @param message the message to check acceptance
 * @exception NullPointerException if message is null
 */
    public boolean acceptMessage(com.topcoder.message.im.Message message) {        
        // your code here
        return false;
    } 

/**
 * <p>Abstract method to initialize the sending connection with merged parameters. It acts as a template method that will be overriden in subclasses.</p>
 * 
 * 
 * 
 * 
 * @param parameters the initialization parameters
 * @exception NullPointerException if parameters is null
 * @exception ConnectionInitializationException if connection can not be initialized
 */
    protected abstract void initializeConnection(java.util.Map parameters);

/**
 * <p>Set the message listener. It will first close the old message listener if there is an old one, and set the new message listener to the listener field. It should invoke increaseReferenceCount() since one listener can be shared among connections.</p>
 * 
 * 
 * 
 * 
 * @param listener the message listener to set
 * @exception NullPointerException if listener is null
 */
    protected void setMessageListener(com.topcoder.message.im.connection.MessageListener listener) {        
        // your code here
    } 

/**
 * <p>Get the message listener. Simply returns the listener field.</p>
 * 
 * 
 * 
 * 
 * @return the message listener
 */
    protected com.topcoder.message.im.connection.MessageListener getMessageListener() {        
        // your code here
        return null;
    } 
 }
